import os
import json
from .client import QRadar


PACKAGE_DIR = os.path.dirname(os.path.abspath(__file__))
CACHE_DIR = os.path.join(PACKAGE_DIR, "cache")

os.makedirs(CACHE_DIR, exist_ok=True)


def cache_read(file):
    if not os.path.exists(file):
        return
    def wrapper(filter):
        with open(file, 'r') as f:
            return json.load(f)
    return wrapper


def cache_write(func, file):
    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        with open(file, 'w') as f:
            json.dump(result, f)
        return result
    return wrapper


class QRadarCached(QRadar):
    def __init__(self, url, key, version, transport, verify=True, file='schema.json'):
        self.file = os.path.join(CACHE_DIR, file)
        super().__init__(url, key, version, transport, verify)

    def api_endpoint_factory(self, method, url):
        default_func = super().api_endpoint_factory(method, url)
        match method, url:
            case "GET", "/help/endpoints":
                return cache_read(self.file) or cache_write(default_func, self.file)
        return default_func



class QRadar:
    def __init__(self, url, key, version, transport, verify=True):
        self.url, self.session = f"{url}/api", transport
        self.session.headers.update(
            {"Accept": "application/json", "Version": version, "SEC": key}
        )
        self.session.verify = verify
        self.version = version

    def __await__(self):
        return self.set_methods().__await__()

    async def set_methods(self):
        self.__dict__.update(
            {
                f"""{(method:=endpoint.get("http_method").lower())}{(path:=endpoint.get("path"))
                .translate({ord('{'):None, ord('}'): None, ord('/'): ord('_')})}""": self.api_endpoint_factory(
                    method.upper(), path
                )
                for endpoint in await self.api_endpoint_factory(
                    "GET", "/help/endpoints"
                )(filter=f"version={self.version}")
            }
        )
        return self

    def api_endpoint_factory(self, method, url):
        async def endpoint(json=None, **params):
            response = await self.session.request(
                method,
                f"{self.url}{url}".format(**params),
                params=params,
                json=json,
            )
            return response.json()
        return endpoint



from setuptools import setup


with open("README.md") as file:
    ld = file.read()

setup(
  name = 'qradar',         
  packages = ['qradar'],
  version = '0.0.3',
  license='MIT',      
  description = 'QRadar client for python with API schema fetching', 
  long_description = ld,
  long_description_content_type = "text/markdown",
  author = 'Chmele',              
  url = 'https://github.com/Chmele/qradar-client',  
  keywords = ['client', 'scripting', 'qradar', 'qradar-api'], 
  requires=[
    "setuptools"
  ],
  classifiers=[
    'Development Status :: 3 - Alpha',    
    'Intended Audience :: Developers',  
    'Topic :: Software Development :: Build Tools',
    'Programming Language :: Python :: 3',   
    'Programming Language :: Python :: 3.10',
    'Programming Language :: Python :: 3.11',
    'Programming Language :: Python :: 3.12',
    'Programming Language :: Python :: 3.13',
  ],
  extras_require={
        "httpx": ["httpx>=0.27"],
        "requests": ["requests>=2.30"]
  },
)


import pytest
from unittest.mock import AsyncMock, MagicMock, Mock
from qradar.async_client import QRadar

@pytest.fixture
def mock_transport():
    transport = Mock()
    transport.headers = {}
    transport.verify = True
    return transport

@pytest.fixture
def qradar_client(mock_transport):
    return QRadar(
        url="https://qradar.example.com",
        key="test-key",
        version="14.0",
        transport=mock_transport,
    )

def test_constructor_initialization(qradar_client):
    assert qradar_client.url == "https://qradar.example.com/api"
    assert qradar_client.version == "14.0"
    assert qradar_client.session.headers == {
        "Accept": "application/json",
        "Version": "14.0",
        "SEC": "test-key",
    }
    assert qradar_client.session.verify is True

class AwaitableMock(AsyncMock):
    def __await__(self):
        self.await_count += 1
        return iter([])

@pytest.mark.asyncio
async def test_set_methods(qradar_client, mock_transport):
    def request_side_effect(method, url, headers=None, params=None, json=None):
        if method == "GET" and url == "https://qradar.example.com/api/help/endpoints":
            return AsyncMock(json=lambda:[
                {"http_method": "GET", "path": "/test/path1"},
                {"http_method": "POST", "path": "/test/path2/{id}"}
            ])
        elif method == "GET" and url == "https://qradar.example.com/api/test/path1":
            return AsyncMock(json=lambda:[{}])
        else:
            AsyncMock(json=lambda:{"unknown"})
    mock_transport.request = AsyncMock()
    mock_transport.request.side_effect = request_side_effect
    
    await qradar_client.set_methods()
    mock_transport.request.assert_awaited_with('GET', 'https://qradar.example.com/api/help/endpoints', params={'filter': 'version=14.0'}, json=None)
    path1 = await qradar_client.get_test_path1(test="test")
    assert path1 == [{}]
    mock_transport.request.assert_awaited_with('GET', 'https://qradar.example.com/api/test/path1', params={'test': 'test'}, json=None)



class QRadar:
    def __init__(self, base_url, key, version, transport, verify=True):
        transport.verify = verify
        self.api_endpoint_factory = lambda method, url: lambda json=None, **params: transport.request(
            method, f"{base_url}/api{url}".format(**params), 
            params=params, json=json,
            headers={"Accept": "application/json", "Version": version, "SEC": key}
        ).json()
        self.__dict__.update({
            f"""{(method:=endpoint.get("http_method").lower())}{(path:=endpoint.get("path"))
            .translate({ord('{'): None, ord('}'): None, ord('/'): ord('_'), ord('-'): ord('_')})}""": self.api_endpoint_factory(
                method.upper(), path
            )
            for endpoint in self.api_endpoint_factory("GET", "/help/endpoints")(
                filter=f"version={version}",
                fields="http_method, path"
            )
        })


import pytest
from unittest.mock import Mock
from oneliner import QRadar

@pytest.fixture
def mock_transport():
    transport = Mock()
    transport.headers = {}
    transport.verify = True
    return transport

@pytest.fixture
def qradar_instance(mock_transport):
    endpoints_response = [
        {"http_method": "GET", "path": "/users"},
        {"http_method": "POST", "path": "/offenses"},
    ]
    mock_transport.request.return_value.json.return_value = endpoints_response
    qradar = QRadar("https://qradar.example.com", "test_key", "12.0", mock_transport)
    return qradar

def test_dynamic_methods_creation(qradar_instance):
    assert hasattr(qradar_instance, "get_users")
    assert hasattr(qradar_instance, "post_offenses")

def test_dynamic_method_calls(qradar_instance, mock_transport):
    def request_side_effect(method, url, headers=None, params=None, json=None):
        if method == "GET" and url == "https://qradar.example.com/api/users":
            return Mock(json=lambda: {"users": ["user1", "user2"]})
        elif method == "POST" and url == "https://qradar.example.com/api/offenses":
            return Mock(json=lambda: {"status": "created"})
        else:
            return Mock(json=lambda: {"error": "unknown endpoint"})

    mock_transport.request.side_effect = request_side_effect

    users = qradar_instance.get_users()
    offenses = qradar_instance.post_offenses(json={"offense": "test"})

    assert users == {"users": ["user1", "user2"]}
    assert offenses == {"status": "created"}

    mock_transport.request.assert_any_call(
        'GET', 'https://qradar.example.com/api/users', params={}, json=None, headers={'Accept': 'application/json', 'Version': '12.0', 'SEC': 'test_key'})
    mock_transport.request.assert_any_call(
        'POST', 'https://qradar.example.com/api/offenses', params={'json': {'offense': 'test'}}, json=None, headers={'Accept': 'application/json', 'Version': '12.0', 'SEC': 'test_key'})

def test_dynamic_method_with_params(qradar_instance, mock_transport):
    def request_side_effect(method, url, headers=None, params=None, json=None):
        if method == "GET" and url == "https://qradar.example.com/api/users":
            return Mock(json=lambda: {"users": ["user1", "user2"]})
        elif method == "POST" and url == "https://qradar.example.com/api/offenses":
            return Mock(json=lambda: {"status": "created"})
        else:
            return Mock(json=lambda: {"error": "unknown endpoint"})

    mock_transport.request.side_effect = request_side_effect

    users = qradar_instance.get_users(filter="active", limit=10)

    assert users == {"users": ["user1", "user2"]}
    print(mock_transport.request.mock_calls)
    mock_transport.request.assert_called_with(
        "GET",
        "https://qradar.example.com/api/users",
        params={"filter": "active", "limit": 10},
        headers={'Accept': 'application/json', 'Version': '12.0', 'SEC': 'test_key'},
        json=None,
    )
